package skyrix;

import java.util.Date;
import java.util.Hashtable;

/**
 * @author <a href="mailto:loutchan@skyrix.com">Eileen Loutchan</a>
 * @version $Revision: $
 */
public class Appointment 
{	
  /**Eindeutige Kennzeichnung von einem Termin.*/
	private String id = null;
  
  /**Bezeichnung fr einen Termin.*/
	private String title = null;
  
  /**Startzeitpunkt eines Termins.*/
	private Date startDate = null;
  
  /**Endzeitpunkt eines Termins.*/
	private Date endDate = null;
	
  /**Konstruktor.
   * 
   * @param id Eindeutige Kennzeichnung.
   * @param title Bezeichnung fr einen Termin.
   * @param startDate Startzeitpunkt eines Termins.
   * @param endDate Endzeitpunkt eines Termins.
   */
	public Appointment(String id, String title, Date startDate, Date endDate)
	{
		super();
	  this.id = id;
	  this.title = title;
	  this.startDate = startDate;
	  this.endDate = endDate;	
	}
	
  /**Gibt die eindeutige ID fr einen Termin zurck.
   * 
   * @return String ID fr einen Termin.
   */
	public String getID()
	{
		return id;
	}

  /**Methode setzt eine eindeutige ID fr einen Termin.
   * 
   * @param id ID die fr diesen Termin gesetzt werden soll.
   */
  public void setID(String id)
  {
    this.id = id;
  }
	
  /**Gibt die Bezeichnung fr einen Termin zurck.
   * 
   * @return String Bezeichnung fr einen Termin.
   */
	public String getTitle()
	{
		return title;
	}

  /**Methode setzt eine Bezeichnung fr einen Termin.
   * 
   * @param title Bezeichnung fr einen Termin.
   */
  public void setTitle(String title)
  {
    this.title = title;
  }
	
  /**Gibt den Startzeitpunkt fr einen Termin zurck.
   * 
   * @return Date Zeitpunkt an dem der Termin beginnt.
   */
	public Date getStartDate()
	{
		return startDate;
	}

  /**Methode setzt den Startzeitpunkt fr einen Termin.
   * 
   * @param startDate Zeitpunkt an dem der Termin beginnen soll.
   */
  public void setStartDate(Date startDate)
  {
    this.startDate = startDate;
  }

  /**Gibt den Endzeitpunkt fr einen Termin zurck.
   *
   * @return Date Zeitpunkt an dem der Termin endet.
   */
	public Date getEndDate()
	{
		return endDate;
	}

  /**Methode setzt den Endzeitpunkt fr einen Termin.
   *
   * @param endDate Zeitpunkt an dem der Termin enden soll.
   */
  public void setEndDate(Date endDate)
  {
    this.endDate = endDate;
  }
	
  /**Methode erstellt ein Termin - Objekt.
   * 
   * @param dataHash Container mit den notwendigen Daten.
   * @return Appointment Termin - Objekt
   */
	public static Appointment createAppointment(Hashtable dataHash)
	{
		try
		{
			String id = (String)dataHash.get("id");
			String title = (String)dataHash.get("title");
			Date startDate = (Date)dataHash.get("startDate");
			Date endDate = (Date)dataHash.get("endDate");
			//Liste mit den Participans durchgehen
			//und Person Objekte erstellen (dataHash.get("participans"))
			return new Appointment(id,
														 title,
														 startDate,
														 endDate);
		}
		catch(Exception ex)
		{
		}
		return null;
	}
  
  /**Stringreprsentation des Objektes.
   * 
   * @return Gibt die Bezeichnung fr einen Termin zurck.
   */
  public String toString()
  {
    return title; 
  }
}
