package skyrix;

import java.io.IOException;
import java.net.URL;
import java.util.Collection;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.Vector;

import org.apache.xmlrpc.XmlRpcClient;
import org.apache.xmlrpc.XmlRpcException;

/**
 * @author <a href="mailto:loutchan@skyrix.com">Eileen Loutchan</a>
 * @version $Revision: $
 */
public class JobManager 
{	
	/**Referenz auf den XML-RPC Client.*/
	private XmlRpcClient client = null;

	/**Konstruktor.
	 * 
	 * <p>Instanziert die Klasse.</p>
	 */
	public JobManager(XmlRpcClient client)
  {
  	this.client = client;			
  }

	/**Methode gibt alle Termine zurck, die auf die bergebenden
	 * Kriterien passen.
	 * 
	 * @param startDate Anfangszeitraum.
	 * @param endDate Endzeitraum.
	 * @return Liste mit den Terminen.
	 */
	public Collection getAllJobs(Date startDate, Date endDate) throws XmlRpcException, IOException
	{
    if(startDate == null) return null;

		LinkedList allJobs = new LinkedList();
		Vector loginv = new Vector();
		if(client.getUserName() == null) return null;
		loginv.addElement(client.getUserName());

		Object loginID = null;
		try
		{
			Object login = client.execute("account.getByLogin", loginv);
			if(login instanceof Hashtable)
			{
				loginID = ((Hashtable)login).get("id");
			}
		}
		catch (IOException ex)
		{
			ex.printStackTrace();
		}
		catch (XmlRpcException ex)
		{
			ex.printStackTrace();
		}

		if(loginID != null)
		{	
			Vector argv = new Vector();
	
			Hashtable hash = new Hashtable();
			Hashtable dateHash = new Hashtable();
			dateHash.put("startDate", startDate);
			if(endDate != null)
			  dateHash.put("endDate", endDate);
	
			hash.put("qualifier", dateHash);
			
	    argv.addElement(loginID);
			argv.addElement("type='toDoJob'");
			argv.addElement(hash);

			Object ret = client.execute("person.fetchJobs", argv);

			if (ret instanceof Vector)
			{
				Vector retVector = (Vector) ret;
				Enumeration en = retVector.elements();
				while (en.hasMoreElements())
				{
					Object data = en.nextElement();
					if (data instanceof Hashtable)
					{
						Job job = Job.createJob((Hashtable) data);
						if (job != null)
							allJobs.add(job);
					}
				}
	       
				System.out.println("getAllJobs(..): "+allJobs);
				return allJobs;
			}
		}
		return null;
	}

	/**Methode gibt alle Job-IDs fr die Jobs, auf die die bergebenden
	 * Kriterien passen, zurck.
	 * 
	 * <p>Die ID wird als String zurckgegeben.</p>
	 * 
	 * @param startDate Anfangszeitraum.
	 * @param endDate Endzeitraum.
	 * @return Liste mit den IDs.
	 */
	public Collection getAllJobIDs(Date startDate, Date endDate) throws XmlRpcException, IOException
	{
    if(startDate == null) return null;

		Vector loginv = new Vector();
		if(client.getUserName() == null) return null;
		loginv.addElement(client.getUserName());

		Object loginID = null;
		try
		{
			Object login = client.execute("account.getByLogin", loginv);
			if(login instanceof Hashtable)
			{
				loginID = ((Hashtable)login).get("id");
			}
		}
		catch (IOException ex)
		{
			ex.printStackTrace();
		}
		catch (XmlRpcException ex)
		{
			ex.printStackTrace();
		}

    if(loginID != null)
    {
			LinkedList allJobs = new LinkedList();
			Vector argv = new Vector();
	
			Hashtable hash = new Hashtable();
			Hashtable dateHash = new Hashtable();
			dateHash.put("startDate", startDate);
			if(endDate != null)
			  dateHash.put("endDate", endDate);
	
			hash.put("qualifier", dateHash);
	
	    argv.addElement(loginID);
	    argv.addElement("type='toDoJob'");
			argv.addElement(hash);
	
			Object ret = client.execute("person.fetchJobIds", argv);

			if (ret instanceof Vector)
			{
				Vector retVector = (Vector) ret;
				Enumeration en = retVector.elements();
				while (en.hasMoreElements())
				{
					Object obj = en.nextElement();
	
					if (obj instanceof String)
					{
						String id = (String) obj;
						if (id != null)
							allJobs.add(id);
					}
				}
	
				System.out.println("getAllJobIDs(..): "+allJobs);
				return allJobs;
			}
    }
    
		return null;
	}

	/**Methode gibt den Job zu der bergebenden ID zurck.
	 * 
	 * @param id Identifiziert einen Job.
	 * @return Job zu der ID.
	 */
	public Job getJobByID(String id) throws XmlRpcException, IOException
	{
		if(id == null) return null;

		Vector loginv = new Vector();
		if(client.getUserName() == null) return null;
		loginv.addElement(client.getUserName());

		Object loginID = null;
		try
		{
			Object login = client.execute("account.getByLogin", loginv);
			if(login instanceof Hashtable)
			{
				loginID = ((Hashtable)login).get("id");
			}
		}
		catch (IOException ex)
		{
			ex.printStackTrace();
		}
		catch (XmlRpcException ex)
		{
			ex.printStackTrace();
		}

		if(loginID != null)
		{			
			Vector argv = new Vector();
			argv.addElement(loginID);
			argv.addElement("type='toDoJob'");
			argv.addElement(id);
	
//			Object ret = client.execute("person.getById", argv);
//	
//			if (ret instanceof Hashtable)
//			{
//				System.out.println("getJobByID(..): "+ret);
//				return Job.createJob((Hashtable) ret);
//			}
		}
		
		return null;
	}

	/**Methode legt einen neuen Job an.
	 * 
	 * @param startDate Anfangszeitpunkt.
	 * @param endDate Endzeitpunkt.
	 * @param name Name des Jobs.
	 * @return Gibt den angelegten Job zurck. 
	 */
	public Job insertJob(Date startDate, Date endDate, String name) throws XmlRpcException, IOException
	{
		if (name.trim().equals("") || startDate == null || endDate == null)
			return null;

		Vector loginv = new Vector();
		if(client.getUserName() == null) return null;
		loginv.addElement(client.getUserName());

		Object loginID = null;
    try
    {
			Object login = client.execute("account.getByLogin", loginv);
			if(login instanceof Hashtable)
			{
				loginID = ((Hashtable)login).get("id");
			}
    }
		catch (IOException ex)
		{
			ex.printStackTrace();
		}
		catch (XmlRpcException ex)
		{
			ex.printStackTrace();
		}

		if(loginID != null)
		{
			
			Vector argv = new Vector();
			Hashtable hash = new Hashtable();
			hash.put("startDate", startDate);
			hash.put("endDate", endDate);
			hash.put("name", name);
			//hash.put("comment", "test");
			argv.addElement(loginID);//eigene ID
			argv.addElement(hash);

			Object ret = client.execute("person.insertJob", argv);

			if (ret instanceof Hashtable)
			{
				System.out.println("insertJob(..): "+ret);
				return Job.createJob((Hashtable) ret);
			}
	  }
		
		return null;
	}

	/**Methode aktualisiert das bergebende Job-Objekt.
	 * 
	 * <p>Das Id Attribut muss zwingend gesetzt sein.</p>
	 * 
	 * @param job Job-Objekt
	 */
	public Job updateJob(Job job) throws XmlRpcException, IOException
	{
		if (job.getID() == null)
			return null;

		Vector loginv = new Vector();
		if(client.getUserName() == null) return null;
		loginv.addElement(client.getUserName());

		Object loginID = null;
		try
		{
			Object login = client.execute("account.getByLogin", loginv);
			if(login instanceof Hashtable)
			{
				loginID = ((Hashtable)login).get("id");
			}
		}
		catch (IOException ex)
		{
			ex.printStackTrace();
		}
		catch (XmlRpcException ex)
		{
			ex.printStackTrace();
		}

		if(loginID != null)
    {
			Vector argv = new Vector();
	    
			Hashtable hash = new Hashtable();
			hash.put("id", job.getID());
			if (job.getStartDate() != null)
				hash.put("startDate", job.getStartDate());
			if (job.getEndDate() != null)
				hash.put("endDate", job.getEndDate());
			if (job.getName() != null)
				hash.put("name", job.getName());
	
			argv.addElement(loginID);//eigene ID
			argv.addElement(hash);
    
    
  		Object ret = client.execute("person.updateJob", argv);

			if (ret instanceof Hashtable)
			{
				System.out.println("updateJob(..): "+ret);
				return Job.createJob((Hashtable) ret);
			}
    }

		return null;
	}

	/**Methode lscht das bergebende Job-Objekt.
	 * 
	 * @param job Job-Objekt
	 */
	public void archiveJob(Job job) throws XmlRpcException, IOException
	{
		if (job.getID() == null)
			return;

		Vector loginv = new Vector();
		if(client.getUserName() == null) return;
		loginv.addElement(client.getUserName());

		Object loginID = null;
		try
		{
			Object login = client.execute("account.getByLogin", loginv);
			if(login instanceof Hashtable)
			{
				loginID = ((Hashtable)login).get("id");
			}
		}
		catch (IOException ex)
		{
			ex.printStackTrace();
		}
		catch (XmlRpcException ex)
		{
			ex.printStackTrace();
		}

		if(loginID != null)
		{
			Vector argv = new Vector();
			Hashtable hash = new Hashtable();
			hash.put("id", job.getID());
			if (job.getStartDate() != null)
				hash.put("startDate", job.getStartDate());
			if (job.getEndDate() != null)
				hash.put("endDate", job.getEndDate());
			if (job.getName() != null)
				hash.put("name", job.getName());
		
	    argv.addElement(loginID);//eigene ID
	    argv.addElement("type='archivedJob'");
			argv.addElement(hash);

      //Job muss archiviert werden
			//Object ret = client.execute("person.archivedJob", argv);
			//System.out.println("Return: "+ret);
		}		
	}

	/**Nur fr die Entwicklung.*/
	public static void main(String[] args)
	{
		try
		{
			XmlRpcClient client = new XmlRpcClient(new URL("http://marvin:30147/RPC2"));
			client.setBasicAuthentication("root", "root");

			JobManager job = new JobManager(client);
			//Job j = job.insertJob(new Date(103, 0, 28), new Date(103, 0, 31), "TestJob 6");
			//job.getAllJobIDs(new Date(103, 0, 28), new Date(103, 0, 29));
			//job.getAllJobs(new Date(103, 0, 28), new Date(103, 0, 29));
			
			//job.getJobByID(j.getID());
			//job.getJobsByIDList(job.getAllJobIDs(new Date(103, 0, 22), new Date(103, 0, 29)));
	    
			//j.setName("Update");
			//job.updateJob(j);
	    
			//job.archivedJob(j);
		}
		catch(Exception e)
		{
		  e.printStackTrace();	
		}
	}
}
