package skyrix;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.net.ConnectException;
import java.net.MalformedURLException;
import java.net.NoRouteToHostException;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.Vector;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.UIManager;

import org.apache.xmlrpc.XmlRpcClient;

import skyrix.gui.LoginDialog;
import skyrix.gui.OrganizerPanel;
import skyrix.gui.lf.BaseLookAndFeel;
import skyrix.gui.lf.DefaultTheme;

/**
 * @author <a href="mailto:loutchan@skyrix.com">Eileen Loutchan</a>
 * @version $Revision: $
 */
public class Organizer
{
	/**Methode ruft den LoginDialog auf und kontaktiert
	 * den XmlRpcClient.
	 * 
	 * @return Gibt eine gltige Instanz vom XmlRpcClient zurck.
	 */
	private XmlRpcClient login()
	{
		boolean doLogin = true;

		while (doLogin)
		{
			LoginDialog loginDialog = new LoginDialog(null, "Login...", true);
			loginDialog.setSize(300, 200);
			center(loginDialog);
			loginDialog.show();

			if (!loginDialog.hasCommitted())
				return null;

			String userName = loginDialog.getUserName();
			String serverURL = loginDialog.getServerURL();
			char[] password = loginDialog.getPassword();

			String errorMessage = null;

			try
			{
				XmlRpcClient client = new XmlRpcClient(new URL(serverURL));
				client.setBasicAuthentication(userName, new String(password));
				client.execute("system.listMethods", new Vector());
				loginDialog.dispose();
				return client;
			}
			catch (MalformedURLException e)
			{
				errorMessage = "The server adress is not a valid url";
			}
			catch (UnknownHostException e)
			{
				errorMessage = "Can't find any host at the given address.";
			}
			catch (NoRouteToHostException e)
			{
				errorMessage =
					"Error connecting to server.\nMaybe you are behind a firewall, proxy or gateway.";
			}
			catch (ConnectException e)
			{
				errorMessage =
					"The server is not a valid Skyrix server or\nSkyrix service is not running.";
			}
			catch (IOException e)
			{
				errorMessage =
					"Invalid user/password combination or server terminated.";
			}
			catch (Exception e)
			{
				errorMessage = "Unable to login to Skyrix server.";
				e.printStackTrace();
			}

			errorMessage += "\nDo you want to try again?";
			if (!(JOptionPane
				.showConfirmDialog(
					null,
					errorMessage,
					"Error...",
					JOptionPane.ERROR_MESSAGE)
				== JOptionPane.OK_OPTION))
			{
				doLogin = false;
			}
		}

		//Fehler
		return null;
	}

	/**Methode zentriert ein Fenster.
	 * 
	 * @param dlg Dialog der zentriert werden soll.
	 */
	public static void center(Dialog dlg)
	{
		Toolkit tool = Toolkit.getDefaultToolkit();
		Dimension screenDim = tool.getScreenSize();
		int screenWidth = screenDim.width;
		int screenHeight = screenDim.height;

		int windowWidth = dlg.getSize().width;
		int windowHeight = dlg.getSize().height;

		int x = (screenWidth - windowWidth) / 2;
		int y = (screenHeight - windowHeight) / 2;

		dlg.setLocation(x, y);
	}

	/**Methode zentriert ein Fenster.
	 * 
	 * @param dlg Dialog der zentriert werden soll.
	 */
	public static void center(Frame frame)
	{
		Toolkit tool = Toolkit.getDefaultToolkit();
		Dimension screenDim = tool.getScreenSize();
		int screenWidth = screenDim.width;
		int screenHeight = screenDim.height;

		int windowWidth = frame.getSize().width;
		int windowHeight = frame.getSize().height;

		int x = (screenWidth - windowWidth) / 2;
		int y = (screenHeight - windowHeight) / 2;

		frame.setLocation(x, y);
	}

	/**Startet die Anwendung.*/
	public static void main(String[] args)
	{

		new Settings();

		/***********************************************/
		//Funktioniert nur unter Java ab Version 1.4
		JFrame.setDefaultLookAndFeelDecorated(true);
		JDialog.setDefaultLookAndFeelDecorated(true);
		/***********************************************/

		try
		{
			if (Settings
				.getLFClass()
				.equalsIgnoreCase("skyrix.gui.lf.BaseLookAndFeel"))
			{
				DefaultTheme theme = new DefaultTheme();
				BaseLookAndFeel.setCurrentTheme(theme);
			}

			try
			{
				UIManager.setLookAndFeel(Settings.getLFClass());
			}
			catch (Exception e)
			{
				System.err.println("Error loading layout, using default layout.");
				DefaultTheme theme = new DefaultTheme();
				BaseLookAndFeel.setCurrentTheme(theme);
				UIManager.setLookAndFeel("skyrix.gui.lf.BaseLookAndFeel");
			}

		}
		catch (Exception e)
		{
			System.err.println("Can't set layout or theme.");
		}

		Organizer organizer = new Organizer();
		XmlRpcClient client = organizer.login();

		if (client == null)
			System.exit(1);

		JFrame f = new JFrame();
		f.addWindowListener(new WindowAdapter()
		{
			public void windowClosed(WindowEvent e)
			{
				System.exit(0);
			}

			public void windowClosing(WindowEvent e)
			{
				System.exit(0);
			}
		});
		f.setTitle("Appointments/Jobs");

		f.getContentPane().setLayout(new BorderLayout());
		f.getContentPane().add(new OrganizerPanel(client));
		f.setSize(400, 300);
		center(f);
		f.show();
	}
}
