package skyrix;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Properties;

import javax.swing.UIManager;

/**Manager fr die Programmeinstellungen.
 * 
 * @author <a href="mailto:loutchan@skyrix.com">Eileen Loutchan</a>
 * @version $Revision: $
 */

public class Settings
{
	private static Properties settings = new Properties();

	public static String LAYOUT_SYSTEM = "System";
	public static String LAYOUT_SKYRIX = "Skyrix";
	public static String LAYOUT_WINDOWS = "Windows";
	public static String LAYOUT_MAC = "MAC";
	public static String LAYOUT_MOTIF = "Motif";
	public static String LAYOUT_METAL = "Metal";

	public Settings()
	{

		//Einstellungen laden
		try
		{
			FileInputStream in = new FileInputStream("organizer.conf");
			settings.load(in);
		}
		catch (Exception e)
		{
			System.err.println("Configfile not found, using default settings.");

			settings.setProperty("showMenu", "0");
			settings.setProperty("refreshTime", "3");
			settings.setProperty("layout", LAYOUT_SKYRIX);
		}
	}

	/**Bestimmt, ob das Men angezeigt werden soll, oder nicht.
	 *
	 * @param showMenu<code>true</code> wenn ein Men angezeigt werden soll,
	 * <code>false</code> sonst.
	 */
	public static void setShowMenu(boolean showMenu)
	{
		settings.setProperty("showMenu", showMenu ? "1" : "0");
	}

	/**Ermittelt, ob das Men angezeigt werden soll, oder nicht.
	 *
	 * @return <code>true</code> wenn ein Men angezeigt werden soll,
	 * <code>false</code> sonst.
	 */
	public static boolean showMenu()
	{
		return (settings.getProperty("showMenu", "0").equalsIgnoreCase("1"));
	}

	/**Setzt die Zeit, in Minuten, die vergehen soll, bis ein automatisches 
	 * Update der Daten vom Server erfolgen soll.
	 * 
	 * @param refreshTime Updateintervall; ein Wert von <code>0</code> bedeutet
	 * kein automatisches Update.
	 */
	public static void setRefreshTime(int refreshTime)
	{
		if (refreshTime < 0)
			return;

		settings.setProperty("refreshTime", "" + refreshTime);
	}

	/**Ermittelt die Zeit, in Minuten, die vergehen soll, bis ein automatisches
	 * Update der Daten vom Server erfolgen soll.
	 *
	 * @return Updateintervall; ein Wert von <code>0</code> bedeutet kein
	 * automatisches Update.
	 */
	public static int getRefreshTime()
	{
		try
		{
			int refreshTime =
				Integer.parseInt(settings.getProperty("refreshTime", "3"));
			if (refreshTime >= 0 && refreshTime <= 60)
				return refreshTime;
		}
		catch (Exception e)
		{
		}

		return 3;
	}

	/**Setzt das Layout, das verwendet werden soll.
	 *
	 * @param layout das Layout; mgliche Werte sind:
	 * 	<ul>
	 * 		<li>{@link #LAYOUT_SYSTEM  LAYOUT_SYSTEM}</li>
	 * 		<li>{@link #LAYOUT_SKYRIX  LAYOUT_SKYRIX}</li>
	 * 		<li>{@link #LAYOUT_WINDOWS LAYOUT_WINDOWS}</li>
	 * 		<li>{@link #LAYOUT_MAC     LAYOUT_MAC}</li>
	 * 		<li>{@link #LAYOUT_MOTIF   LAYOUT_MOTIF}</li>
	 * 		<li>{@link #LAYOUT_METAL   LAYOUT_METAL}</li>
	 * 	</ul>
	 */
	public static void setLayout(String layout)
	{
		if (layout == null)
			return;
		settings.setProperty("layout", layout);
	}

	/**Setzt das Layout, das verwendet werden soll.
	 *
	 * @param layout das Layout; mgliche Werte sind:
	 *  <ul>
	 *    <li>{@link #LAYOUT_SYSTEM  LAYOUT_SYSTEM}</li>
	 *    <li>{@link #LAYOUT_SKYRIX  LAYOUT_SKYRIX}</li>
	 *    <li>{@link #LAYOUT_WINDOWS LAYOUT_WINDOWS}</li>
	 *    <li>{@link #LAYOUT_MAC     LAYOUT_MAC}</li>
	 *    <li>{@link #LAYOUT_MOTIF   LAYOUT_MOTIF}</li>
	 *    <li>{@link #LAYOUT_METAL   LAYOUT_METAL}</li>
	 *  </ul>
	 */
	public static String getLayout()
	{
		return settings.getProperty("layout", LAYOUT_SKYRIX);
	}

	/**Ermittelt die Klasse, die als Look and Feel gesetzt werden soll.
	 * 
	 * @return String
	 */
	public static String getLFClass()
	{
		String layout = getLayout();
		if (layout.equalsIgnoreCase(LAYOUT_SYSTEM))
			return UIManager.getSystemLookAndFeelClassName();
		else if (layout.equalsIgnoreCase(LAYOUT_WINDOWS))
			return "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
		else if (layout.equalsIgnoreCase(LAYOUT_MAC))
			return "javax.swing.plaf.mac.MacLookAndFeel";
		else if (layout.equalsIgnoreCase(LAYOUT_MOTIF))
			return "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
		else if (layout.equalsIgnoreCase(LAYOUT_METAL))
			return "javax.swing.plaf.metal.MetalLookAndFeel";

		//Voreinstellung
		return "skyrix.gui.lf.BaseLookAndFeel";
	}

	/**Speichert die Einstellung in der Konfigurationsdatei (organizer.conf) ab.
	 */
	public static void saveSettings()
	{
		try
		{
			FileOutputStream out = new FileOutputStream("organizer.conf");
			settings.store(out, "Skyrix config file");
		}
		catch (Exception e)
		{
			System.err.println("Error saving settings: " + e.getMessage());
		}
	}
}
