package skyrix.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.net.URL;
import java.text.DateFormat;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.Locale;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SpinnerDateModel;
import javax.swing.Timer;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;

import org.apache.xmlrpc.XmlRpcClient;

import skyrix.Appointment;
import skyrix.AppointmentManager;
import skyrix.Job;
import skyrix.JobManager;
import skyrix.Settings;

/**
 * @author <a href="mailto:loutchan@skyrix.com">Eileen Loutchan</a>
 * @version $Revision: $
 */
public class OrganizerPanel extends JPanel
{
	/**Instanz von dem Terminmanager.*/
	private AppointmentManager appManager = null;

	/**Instanz von dem Jobmanager.*/
	private JobManager jobManager = null;

	/**Bezeichnung fr einen Termin oder Job.*/
	private JTextField title = null;

	/**Feld fr die nhere Erluterung
	 * zu einem Termin oder Job.*/
	private JTextArea comment = null;

	/**Zeitpunkt zu dem ein Ereignis beginnen soll.*/
	private JSpinner startDate = null;

	/**Zeitpunkt an dem ein Ereignis endet.*/
	private JSpinner endDate = null;

	/**Tabelle zeigt alle gltigen Jobs an.*/
	private JTable jobTable = null;

	/**Tabelle zeigt alle gltigen Termine an.*/
	private JTable appointmentTable = null;

	/**Schaltflche zum Lschen eines Termins.*/
	private HoverButton removeAppButton = null;

	/**SelectionModel fr die Termin Tabelle.*/
	private ListSelectionModel appSelectionModel = null;

	/**Schaltflche zum Lschen eines Jobs.*/
	private HoverButton removeJobButton = null;

	/**SelectionModel fr die Job Tabelle.*/
	private ListSelectionModel jobSelectionModel = null;

	/**Timer kontaktiert alle 5 Minuten den Server, um die 
	 * Daten zuaktualisieren.*/
	private final Timer timer;

	/**Url fr die Serververbindung.*/
	private String serverUrl = null;

	/**Anmeldename.*/
	private String loginName = null;

	/**Passwort.*/
	private char[] password = null;

	/**Konstruktor.
	 * 
	 * <p>Erstellt die GUI.</p>
	 */
	public OrganizerPanel(XmlRpcClient client)
	{
		super();
		setLayout(new BorderLayout());

		appManager = new AppointmentManager(client);
		jobManager = new JobManager(client);

		//Panel zum Anlegen eines neuen Termins oder Jobs
		JPanel newEntry = new JPanel();

		JLabel titleLabel = new JLabel("Title:");
		title = new JTextField();

		JLabel commentLabel = new JLabel("Comment:");
		comment = new JTextArea(5, 0);
		JScrollPane commentPane = new JScrollPane(comment);

		JLabel startLabel = new JLabel("Start date:");
		startDate = new JSpinner();
		startDate.setModel(
			new SpinnerDateModel(
				new Date(),
				null,
				null,
				Calendar.DAY_OF_WEEK_IN_MONTH));

		JLabel endLabel = new JLabel("End date:");
		Calendar calendar = Calendar.getInstance(Locale.GERMAN);
		endDate = new JSpinner();
		endDate.setModel(
			new SpinnerDateModel(
				new Date(),
				null,
				null,
				Calendar.DAY_OF_YEAR));

		GridBagLayout gbl = new GridBagLayout();
		newEntry.setLayout(gbl);

		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(0, 5, 10, 15); // Abstnde definieren
		c.weightx = 10; // Breite
		c.weighty = 10; // Hhe
		c.gridx = 0; // x-Position
		c.gridy = 0; // y-Position
		c.gridwidth = 1; // Breite
		c.gridheight = 1; // Hhe
		c.fill = GridBagConstraints.HORIZONTAL;
		// wie Komponente Bereich fllen soll
		gbl.setConstraints(titleLabel, c); //zum GridBagConstraints hinzufgen
		newEntry.add(titleLabel);

		c.weightx = 90; // Breite
		c.weighty = 0; // Hhe
		c.gridx = 1;
		c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 1;
		gbl.setConstraints(title, c);
		newEntry.add(title);

		c.weightx = 0; // Breite
		c.weighty = 70; // Hhe
		c.gridx = 0;
		c.gridy = 1;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.anchor = GridBagConstraints.NORTHWEST;
		gbl.setConstraints(commentLabel, c);
		newEntry.add(commentLabel);

		c.weightx = 0;
		c.weighty = 0; // Breite
		c.gridx = 1;
		c.gridy = 1;
		c.gridwidth = 1;
		c.gridheight = 2;
		c.fill = GridBagConstraints.BOTH;
		gbl.setConstraints(commentPane, c);
		newEntry.add(commentPane);

		c.weighty = 10;
		c.gridx = 0;
		c.gridy = 3;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		gbl.setConstraints(startLabel, c);
		newEntry.add(startLabel);

		c.weighty = 0;
		c.gridx = 1;
		c.gridy = 3;
		c.gridwidth = 1;
		c.gridheight = 1;
		gbl.setConstraints(startDate, c);
		newEntry.add(startDate);

		c.weighty = 10;
		c.gridx = 0;
		c.gridy = 4;
		c.gridwidth = 1;
		c.gridheight = 1;
		gbl.setConstraints(endLabel, c);
		newEntry.add(endLabel);

		c.weighty = 0;
		c.gridx = 1;
		c.gridy = 4;
		c.gridwidth = 1;
		c.gridheight = 1;
		gbl.setConstraints(endDate, c);
		newEntry.add(endDate);

		URL appImageFile = getClass().getResource("images/appointment.gif");
		URL jobImageFile = getClass().getResource("images/jobs.gif");
		URL deleteImageFile = getClass().getResource("images/trash.gif");
		URL reloadImageFile = getClass().getResource("images/reload.gif");

		HoverButton newAppointment = null;
		if (appImageFile != null && jobImageFile != null)
		{
			newAppointment = new HoverButton(new ImageIcon(appImageFile));
		}
		else
			newAppointment = new HoverButton("Appointment");

		newAppointment.setToolTipText("Create appointment");
		newAppointment.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				createNewAppointment();
			}
		});

		HoverButton newJob = null;
		if (appImageFile != null && jobImageFile != null)
		{
			newJob = new HoverButton(new ImageIcon(jobImageFile));
		}
		else
			newJob = new HoverButton("Job");

		newJob.setToolTipText("Create job");
		newJob.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				createNewJob();
			}
		});

		JPanel buttonPanel1 = new JPanel(new GridLayout(1, 2));
		buttonPanel1.add(newAppointment);
		buttonPanel1.add(newJob);

		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		buttonPanel.add(buttonPanel1);

		JPanel newEntryPanel = new JPanel(new BorderLayout());
		newEntryPanel.add(newEntry, BorderLayout.CENTER);
		newEntryPanel.add(buttonPanel, BorderLayout.SOUTH);

    //Panel zum Anzeigen aller Termine
		TitledBorder border = null;

		JPanel aptTablePanel = new JPanel(new BorderLayout());

		JPanel appTableButtonPanel1 = new JPanel(new GridLayout(0, 2));

		if (deleteImageFile != null && reloadImageFile != null)
		{
			ImageIcon icon = new ImageIcon(deleteImageFile);
			System.out.println("Hhe: "+icon.getIconHeight());
			System.out.println("Breite: "+icon.getIconWidth());

			removeAppButton = new HoverButton(new ImageIcon(deleteImageFile));
		}
		else
			removeAppButton = new HoverButton("Delete");

    //removeAppButton.setPreferredSize(new Dimension(icon.getIconWidth(), icon.getIconHeight()));

		removeAppButton.setToolTipText("Delete");
		removeAppButton.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				removeAppointment();
			}
		});
		removeAppButton.setEnabled(false);

		HoverButton refreshAppButton = null;
		if (deleteImageFile != null && reloadImageFile != null)
		{
			refreshAppButton = new HoverButton(new ImageIcon(reloadImageFile));
		}
		else
			refreshAppButton = new HoverButton("Refresh");

		//icon = new ImageIcon(reloadImageFile);
		//refreshAppButton.setPreferredSize(new Dimension(icon.getIconWidth(), icon.getIconHeight()));

		refreshAppButton.setToolTipText("Refresh");
		refreshAppButton.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				initializeAppointments();
				if (Settings.getRefreshTime() > 0)
					timer.restart();
			}
		});

		appTableButtonPanel1.add(refreshAppButton);
		appTableButtonPanel1.add(removeAppButton);

		JPanel appTableButtonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		appTableButtonPanel.add(appTableButtonPanel1);
		
		aptTablePanel.add(appTableButtonPanel, BorderLayout.NORTH);

		DefaultTableModel appTableModel = new DefaultTableModel()
		{
			public boolean isCellEditable(int row, int column)
			{
				return false;
			}
		};

		appointmentTable = new JTable(appTableModel);
		appointmentTable.setSelectionMode(
			ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		appointmentTable.addMouseListener(new MouseAdapter()
		{
			public void mousePressed(MouseEvent e)
			{
				if (e.getButton() == MouseEvent.BUTTON3)
					appointmentClicked(e);
			}
		});

		appSelectionModel =
			(ListSelectionModel) appointmentTable.getSelectionModel();
		appSelectionModel.addListSelectionListener(new ListSelectionListener()
		{
			public void valueChanged(ListSelectionEvent e)
			{
				appTableSelection();
			}
		});

		//initialisiert die Termine
		initializeAppointments();

		JScrollPane appointmentPane = new JScrollPane(appointmentTable);
		border = BorderFactory.createTitledBorder("");
		aptTablePanel.setBorder(border);
		aptTablePanel.add(appointmentPane, BorderLayout.CENTER);
		aptTablePanel.setPreferredSize(new Dimension(200, 100));

    //Panel zum Anzeigen aller Jobs
		JPanel jobTablePanel = new JPanel(new BorderLayout());

		JPanel jobTableButtonPanel1 = new JPanel(new GridLayout(0, 2));

		if (deleteImageFile != null && reloadImageFile != null)
		{
			removeJobButton = new HoverButton(new ImageIcon(deleteImageFile));
		}
		else
			removeJobButton = new HoverButton("Archive");

		removeJobButton.setToolTipText("Archive");
		removeJobButton.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				removeJob();
			}
		});
		removeJobButton.setEnabled(false);

		HoverButton refreshJobButton = null;
		if (deleteImageFile != null && reloadImageFile != null)
		{
			refreshJobButton = new HoverButton(new ImageIcon(reloadImageFile));
		}
		else
			refreshJobButton = new HoverButton("Refresh");

		refreshJobButton.setToolTipText("Refresh");
		refreshJobButton.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				initializeJobs();
				if (Settings.getRefreshTime() > 0)
					timer.restart();
			}
		});

		jobTableButtonPanel1.add(refreshJobButton);
		jobTableButtonPanel1.add(removeJobButton);

		JPanel jobTableButtonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		jobTableButtonPanel.add(jobTableButtonPanel1);

		jobTablePanel.add(jobTableButtonPanel, BorderLayout.NORTH);

		DefaultTableModel jobTableModel = new DefaultTableModel()
		{
			public boolean isCellEditable(int row, int column)
			{
				return false;
			}
		};

		jobTable = new JTable(jobTableModel);
		jobTable.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		jobTable.addMouseListener(new MouseAdapter()
		{
			public void mousePressed(MouseEvent e)
			{
				if (e.getButton() == MouseEvent.BUTTON3)
					jobClicked(e);
			}
		});

		jobSelectionModel = (ListSelectionModel) jobTable.getSelectionModel();
		jobSelectionModel.addListSelectionListener(new ListSelectionListener()
		{
			public void valueChanged(ListSelectionEvent e)
			{
				jobTableSelection();
			}
		});

		//initialisiert die Jobs
		initializeJobs();

		JScrollPane jobPane = new JScrollPane(jobTable);
		border = BorderFactory.createTitledBorder("");
		jobTablePanel.setBorder(border);
		jobTablePanel.add(jobPane, BorderLayout.CENTER);
		jobTablePanel.setPreferredSize(new Dimension(200, 100));

		//TabbedPane fasst alle Ansichten zusammen
		JTabbedPane tabPanel = new JTabbedPane(JTabbedPane.TOP);
		tabPanel.addTab("New", newEntryPanel);
		tabPanel.addTab("Appointments", aptTablePanel);
		tabPanel.addTab("Jobs", jobTablePanel);
		tabPanel.addTab("Settings", new SettingsPanel(OrganizerPanel.this));

		add(tabPanel, BorderLayout.CENTER);
		Timer t = new Timer(Settings.getRefreshTime() * 60000, new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				initializeAppointments();
				initializeJobs();
			}
		});
		if (Settings.getRefreshTime() > 0)
			t.start();
		timer = t;
	}

	/**Methode erstellt einen neuen Job.
	 *
	 * <p>Der Job wird in der Datenbank angelegt und 
	 * in einer Liste eingetragen.</p>
	 */
	private void createNewJob()
	{
		DateFormat dateFormat = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.SHORT, Locale.GERMAN);
		SpinnerDateModel startDateModel = (SpinnerDateModel) startDate.getModel();
		Date sDate = startDateModel.getDate();

		SpinnerDateModel endDateModel = (SpinnerDateModel) endDate.getModel();
		Date eDate = endDateModel.getDate();
		String formatedEndDate = dateFormat.format(eDate);

		String name = null;
		if (title.getText().trim().equals(""))
		{
			JOptionPane.showMessageDialog(
				this,
				"You must fill the title field.",
				"Information...",
				JOptionPane.INFORMATION_MESSAGE);
		}
		else
			name = title.getText();

		if (name == null)
			return;

		if (sDate.compareTo(startDateModel.getValue()) < 0
			|| eDate.compareTo(sDate) < 0)
		{
			JOptionPane.showMessageDialog(
				this,
				"The startdate can not be in the past\n"
					+ "and the enddate must be greater than the startdate.",
				"Information...",
				JOptionPane.INFORMATION_MESSAGE);
			return;
		}

		try
		{
			Job job = jobManager.insertJob(sDate, eDate, name);
			if (job != null)
			{
				Vector v = new Vector();
				v.addElement(job);
				v.addElement(formatedEndDate);
				((DefaultTableModel) jobTable.getModel()).addRow(v);
			}
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(
				this,
				"Can't create new job.",
				"Error...",
				JOptionPane.ERROR_MESSAGE);
		}
	}

	/**Methode erstellt einen neuen Termin.
	 * 
	 * <p>Der Termin wird in der Datenbank angelegt und 
	 * in einer Liste eingetragen.</p>
	 */
	private void createNewAppointment()
	{
		DateFormat dateFormat = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.SHORT, Locale.GERMAN);
		SpinnerDateModel startDateModel = (SpinnerDateModel) startDate.getModel();
		Date sDate = startDateModel.getDate();
		String formatedStartDate = dateFormat.format(sDate);

		SpinnerDateModel endDateModel = (SpinnerDateModel) endDate.getModel();
		Date eDate = endDateModel.getDate();

		String name = null;
		if (title.getText().trim().equals(""))
		{
			JOptionPane.showMessageDialog(
				this,
				"You must fill the title field.",
				"Information...",
				JOptionPane.INFORMATION_MESSAGE);
		}
		else
			name = title.getText();

		if (name == null)
			return;

		if (sDate.compareTo(startDateModel.getValue()) < 0
			|| eDate.compareTo(sDate) < 0)
		{
			JOptionPane.showMessageDialog(
				this,
				"The startdate can not be in the past\n"
					+ "and the enddate must be greater than the startdate.",
				"Information...",
				JOptionPane.INFORMATION_MESSAGE);
			return;
		}

		String commentText = comment.getText();

		try
		{
			Appointment appointment =
				appManager.insertAppointment(sDate, eDate, name, commentText);
			if (appointment != null)
			{
				Vector v = new Vector();
				v.addElement(appointment);
				v.addElement(formatedStartDate);
				((DefaultTableModel) appointmentTable.getModel()).addRow(v);
			}
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(
				this,
				"Can't create new appointment.",
				"Error...",
				JOptionPane.ERROR_MESSAGE);
		}
	}

	/**Methode wird beim Klick mit der rechten Maustaste
	 * in die Termintabelle aufgerufen, 
	 * um die markierten Termine zulschen
	 */
	private void appointmentClicked(MouseEvent e)
	{
		JPopupMenu popup = new JPopupMenu();
		JMenuItem remove = new JMenuItem("Remove");
		remove.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				removeAppointment();
			}
		});
		popup.add(remove);
		popup.show(appointmentTable, e.getX(), e.getY());
	}

	/**Methode wird beim Klick mit der rechten Maustaste
	 * in die Jobtabelle aufgerufen,
	 * um die markierten Jobs zulschen
	 */
	private void jobClicked(MouseEvent e)
	{
		JPopupMenu popup = new JPopupMenu();
		JMenuItem remove = new JMenuItem("Remove");
		remove.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				removeJob();
			}
		});
		popup.add(remove);
		popup.show(appointmentTable, e.getX(), e.getY());
	}

	/**Methode lscht alle ausgewhlten Termine.
	 */
	private void removeAppointment()
	{
		DefaultTableModel model = ((DefaultTableModel) appointmentTable.getModel());
		int[] rows = appointmentTable.getSelectedRows();
		for (int i = rows.length - 1; i >= 0; i--)
		{
			try
			{
				int rem = rows[i];
				Object appointment = model.getValueAt(rem, 0);

				if (appointment instanceof Appointment)
				{
					appManager.deleteAppointment((Appointment) appointment);
					model.removeRow(rem);
				}
			}
			catch (Exception e)
			{
				JOptionPane.showMessageDialog(
					this,
					"Can't delete the selected appointments.",
					"Error...",
					JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	/**Methode lscht alle ausgewhlten Jobs.
	 */
	private void removeJob()
	{
		DefaultTableModel model = ((DefaultTableModel) jobTable.getModel());
		int[] rows = jobTable.getSelectedRows();
		for (int i = rows.length - 1; i >= 0; i--)
		{
			try
			{
				int rem = rows[i];
				Object job = model.getValueAt(rem, 0);

				if (job instanceof Job)
				{
					jobManager.archiveJob((Job) job);
					model.removeRow(rem);
				}
			}
			catch (Exception e)
			{
				JOptionPane.showMessageDialog(
					this,
					"Can't archive the selected jobs.",
					"Error...",
					JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	/**Methode wird aufgerufen, wenn sich die Auswahl in der
	 * Termin Table gendert hat.
	 */
	private void appTableSelection()
	{
		removeAppButton.setEnabled(!appSelectionModel.isSelectionEmpty());
	}

	/**Methode wird aufgerufen, wenn sich die Auswahl in der
	 * Job Table gendert hat.
	 */
	private void jobTableSelection()
	{
		//removeJobButton.setEnabled(!jobSelectionModel.isSelectionEmpty());
		removeJobButton.setEnabled(false);
	}

	public void configure()
	{
		timer.stop();
		timer.setDelay(Settings.getRefreshTime() * 60000);
		if (Settings.getRefreshTime() > 0) timer.start();
	}

	/**Methode holt alle gltigen Termine vom Server
	 * und stellt sie in der Tabelle dar.*/
	private synchronized void initializeAppointments()
	{
		Vector rows = new Vector();

		Vector columnNames = new Vector();
		columnNames.addElement("Name");
		columnNames.addElement("Date");

		try
		{
			DateFormat dateFormat = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.SHORT, Locale.GERMAN);

			Object ret = appManager.getAllAppointments(new Date(), null);
			if (ret instanceof Collection)
			{
				Iterator appList = ((Collection) ret).iterator();
				while (appList.hasNext())
				{
					Object obj = appList.next();
					if (obj instanceof Appointment)
					{
						Appointment appointment = (Appointment) obj;
						String formatedStartDate =
							dateFormat.format(appointment.getStartDate());
						Vector v = new Vector();
						v.addElement(appointment);
						v.addElement(formatedStartDate);

						rows.addElement(v);
					}
				}
			}
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(
				this,
				"Can't initialize appointments.",
				"Error...",
				JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
		}

		((DefaultTableModel) appointmentTable.getModel()).setDataVector(
			rows,
			columnNames);
	}

	/**Methode holt alle gltigen Jobs vom Server
	 * und stellt sie in der Tabelle dar.
	 */
	private synchronized void initializeJobs()
	{
		Vector rows = new Vector();

		Vector columnNames = new Vector();
		columnNames.addElement("Name");
		columnNames.addElement("Date");

		try
		{
			DateFormat dateFormat = DateFormat.getDateInstance(DateFormat.MEDIUM, Locale.GERMAN);

			Object ret = jobManager.getAllJobs(new Date(), null);
			if (ret instanceof Collection)
			{
				Iterator jobList = ((Collection) ret).iterator();
				while (jobList.hasNext())
				{
					Object obj = jobList.next();
					if (obj instanceof Job)
					{
						Job job = (Job) obj;
						
						String formatedEndDate = dateFormat.format(job.getEndDate());
						Vector v = new Vector();
						v.addElement(job);
						v.addElement(formatedEndDate);

						rows.addElement(v);
					}
				}
			}
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(
				this,
				"Can't initialize jobs.",
				"Error...",
				JOptionPane.ERROR_MESSAGE);
		}

		((DefaultTableModel) jobTable.getModel()).setDataVector(rows, columnNames);
	}
}
