package skyrix.gui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import skyrix.Settings;

/**Dialog fr die Programmeinstellungen.
 * 
 * @author <a href="mailto:loutchan@skyrix.com">Eileen Loutchan</a>
 * @version $Revision: $
 */
public class SettingsPanel extends JPanel
{
	/**Aktualisierungsrate.*/
	private JSpinner refresh = null;

	/**Layout.*/
	private JComboBox layoutBox = null;

	/**Men.*/
	private JCheckBox menuBox = null;

  /**Instanz vom OrganizerPanel.*/
  private OrganizerPanel orgPanel = null;

	/**Konstruktor.
	 * 
	 * <p>Erstellt die GUI.</p>
	 * 
	 * @param owner Elternfenster der Komponente.
	 * @param title Titel fr den Dialog
	 * @param modal True der Dialog bleibt im Vordergrund.
	 */
	public SettingsPanel(OrganizerPanel orgPanel)
	{
		this.orgPanel = orgPanel;
		setLayout(new BorderLayout());

		JPanel loginPanel = new JPanel();
		JLabel refreshLabel = new JLabel("Refresh rate [min] (0 = off):");
		refresh = new JSpinner();
		int value = Settings.getRefreshTime();
		refresh.setModel(new SpinnerNumberModel(value, 0, 60, 1));

		JLabel lfLabel = new JLabel("Look and feel:");
		layoutBox =
			new JComboBox(
				new String[] {
					"System",
					"Skyrix",
					"Windows",
					"Mac",
					"Motif",
					"Metal" });
		layoutBox.setSelectedItem(Settings.getLayout());

		menuBox = new JCheckBox("Show menubar", Settings.showMenu());

		menuBox.setVisible(false);

		GridBagLayout gbl = new GridBagLayout();
		loginPanel.setLayout(gbl);

		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(10, 5, 10, 15);
		c.weightx = 10;
		c.weighty = 0;
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		gbl.setConstraints(refreshLabel, c);
		loginPanel.add(refreshLabel);

		c.weightx = 90;
		c.gridx = 1;
		gbl.setConstraints(refresh, c);
		loginPanel.add(refresh);

		c.weightx = 0;
		c.gridx = 0;
		c.gridy = 1;
		gbl.setConstraints(lfLabel, c);
		loginPanel.add(lfLabel);

		c.gridx = 1;
		gbl.setConstraints(layoutBox, c);
		loginPanel.add(layoutBox);

		c.gridx = 0;
		c.gridy = 3;
		gbl.setConstraints(menuBox, c);
		loginPanel.add(menuBox);

		URL saveImageFile = getClass().getResource("images/ok.gif");

		HoverButton save = null;
		if (saveImageFile != null)
		{
			save = new HoverButton("Save", new ImageIcon(saveImageFile));
		}
		else
			save = new HoverButton("Save");

		save.setToolTipText("Save");
		save.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				saveButtonClicked();
			}
		});

		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		buttonPanel.add(save);

		add(loginPanel, BorderLayout.NORTH);
		add(buttonPanel, BorderLayout.SOUTH);
	}

	/**Methode wird aufgerufen, um sich mit dem Server
	 * zuverbinden.*/
	private void saveButtonClicked()
	{
		try
		{
			Settings.setLayout(layoutBox.getSelectedItem().toString());
			Settings.setRefreshTime(
				((Integer) refresh.getModel().getValue()).intValue());
			Settings.setShowMenu(menuBox.isSelected());
			Settings.saveSettings();

			JOptionPane.showMessageDialog(
				this,
				"Layout setting will take effect after restart.",
				"Information...",
				JOptionPane.INFORMATION_MESSAGE);
			orgPanel.configure();
		}
		catch (Exception e)
		{
		}
	}
}
